@@ -271,8 +271,9 @@
 	static block gen_offset(uint64_t KtopStr[3], unsigned bot) {
 		const union { unsigned x; unsigned char endian; } little = { 1 };
 		const int64x2_t k64 = {-64,-64};
-		uint64x2_t hi = *(uint64x2_t *)(KtopStr+0);   /* hi = A B */
-		uint64x2_t lo = *(uint64x2_t *)(KtopStr+1);   /* hi = B C */
+		uint64x2_t hi, lo;
+		memcpy(&hi, KtopStr, sizeof(hi));
+		memcpy(&lo, KtopStr+1, sizeof(lo));
 		int64x2_t ls = vdupq_n_s64(bot);
 		int64x2_t rs = vqaddq_s64(k64,ls);
 		block rval = (block)veorq_u64(vshlq_u64(hi,ls),vshlq_u64(lo,rs));
@@ -390,7 +391,7 @@ static inline void AES_set_encrypt_key(unsigned char *handle, const int bits, AE
 {
 	CCCryptorStatus rv = CCCryptorCreateFromData(
 		kCCEncrypt,
-		kCCAlgorithmAES,
+		kCCAlgorithmAES128,
 		kCCOptionECBMode,
 		handle,
 		bits / 8,
@@ -406,7 +407,7 @@ static inline void AES_set_decrypt_key(unsigned char *handle, const int bits, AE
 {
 	CCCryptorStatus rv = CCCryptorCreateFromData(
 		kCCDecrypt,
-		kCCAlgorithmAES,
+		kCCAlgorithmAES128,
 		kCCOptionECBMode,
 		handle,
 		bits / 8,
@@ -963,13 +964,20 @@ static void process_ad(ae_ctx *ctx, const void *ad, int ad_len, int final)
 			switch (k) {
 				#if (BPI == 8)
 				case 8: ad_checksum = xor_block(ad_checksum, ta[7]);
+					/* fallthrough */
 				case 7: ad_checksum = xor_block(ad_checksum, ta[6]);
+					/* fallthrough */
 				case 6: ad_checksum = xor_block(ad_checksum, ta[5]);
+					/* fallthrough */
 				case 5: ad_checksum = xor_block(ad_checksum, ta[4]);
+					/* fallthrough */
 				#endif
 				case 4: ad_checksum = xor_block(ad_checksum, ta[3]);
+					/* fallthrough */
 				case 3: ad_checksum = xor_block(ad_checksum, ta[2]);
+					/* fallthrough */
 				case 2: ad_checksum = xor_block(ad_checksum, ta[1]);
+					/* fallthrough */
 				case 1: ad_checksum = xor_block(ad_checksum, ta[0]);
 			}
 			ctx->ad_checksum = ad_checksum;
@@ -1131,12 +1139,18 @@ int ae_encrypt(ae_ctx     *  ctx,
 		switch (k) {
 			#if (BPI == 8)
 			case 7: ctp[6] = xor_block(ta[6], oa[6]);
+				/* fallthrough */
 			case 6: ctp[5] = xor_block(ta[5], oa[5]);
+				/* fallthrough */
 			case 5: ctp[4] = xor_block(ta[4], oa[4]);
+				/* fallthrough */
 			case 4: ctp[3] = xor_block(ta[3], oa[3]);
+				/* fallthrough */
 			#endif
 			case 3: ctp[2] = xor_block(ta[2], oa[2]);
+				/* fallthrough */
 			case 2: ctp[1] = xor_block(ta[1], oa[1]);
+				/* fallthrough */
 			case 1: ctp[0] = xor_block(ta[0], oa[0]);
 		}
 
@@ -1335,17 +1349,23 @@ int ae_decrypt(ae_ctx     *ctx,
 			#if (BPI == 8)
 			case 7: ptp[6] = xor_block(ta[6], oa[6]);
 				    checksum = xor_block(checksum, ptp[6]);
+				    /* fallthrough */
 			case 6: ptp[5] = xor_block(ta[5], oa[5]);
 				    checksum = xor_block(checksum, ptp[5]);
+				    /* fallthrough */
 			case 5: ptp[4] = xor_block(ta[4], oa[4]);
 				    checksum = xor_block(checksum, ptp[4]);
+				    /* fallthrough */
 			case 4: ptp[3] = xor_block(ta[3], oa[3]);
 				    checksum = xor_block(checksum, ptp[3]);
+				    /* fallthrough */
 			#endif
 			case 3: ptp[2] = xor_block(ta[2], oa[2]);
 				    checksum = xor_block(checksum, ptp[2]);
+				    /* fallthrough */
 			case 2: ptp[1] = xor_block(ta[1], oa[1]);
 				    checksum = xor_block(checksum, ptp[1]);
+				    /* fallthrough */
 			case 1: ptp[0] = xor_block(ta[0], oa[0]);
 				    checksum = xor_block(checksum, ptp[0]);
 		}
