@@ -42,7 +42,6 @@
 #include <wchar.h>
 #include <assert.h>
 #include <wctype.h>
-#include <iostream>
 #include <typeinfo>
 #include <termios.h>
 
@@ -158,8 +157,6 @@ static void emulate_terminal( int fd )
       if ( vt_parser( fd, &parser ) < 0 ) {
 	return;
       }
-    } else if ( sel.error( STDIN_FILENO ) || sel.error( fd ) ) {
-      return;
     } else {
       fprintf( stderr, "select mysteriously woken up\n" );
     }
@@ -195,9 +192,10 @@ static int vt_parser( int fd, Parser::UTF8Parser *parser )
   }
 
   /* feed to parser */
+  Parser::Actions actions;
   for ( int i = 0; i < bytes_read; i++ ) {
-    std::list<Parser::Action *> actions = parser->input( buf[ i ] );
-    for ( std::list<Parser::Action *>::iterator j = actions.begin();
+    parser->input( buf[ i ], actions );
+    for ( Parser::Actions::iterator j = actions.begin();
 	  j != actions.end();
 	  j++ ) {
 
@@ -218,6 +216,7 @@ static int vt_parser( int fd, Parser::UTF8Parser *parser )
 
       fflush( stdout );
     }
+    actions.clear();
   }
 
   return 0;
