@@ -32,6 +32,7 @@
 
 #include "config.h"
 
+#include <err.h>
 #include <errno.h>
 #include <locale.h>
 #include <string.h>
@@ -254,6 +255,10 @@ void STMClient::main_init( void )
 
   /* tell server the size of the terminal */
   network->get_current_state().push_back( Parser::Resize( window_size.ws_col, window_size.ws_row ) );
+
+  /* be noisy as necessary */
+  network->set_verbose( verbose );
+  Select::set_verbose( verbose );
 }
 
 void STMClient::output_new_frame( void )
@@ -407,6 +412,15 @@ bool STMClient::main( void )
   /* initialize signal handling and structures */
   main_init();
 
+  /* Drop unnecessary privileges */
+#ifdef HAVE_PLEDGE
+  /* OpenBSD pledge() syscall */
+  if ( pledge( "stdio inet tty", NULL )) {
+    perror( "pledge() failed" );
+    exit( 1 );
+  }
+#endif
+
   /* prepare to poll for events */
   Select &sel = Select::get_instance();
 
@@ -525,15 +539,16 @@ bool STMClient::main( void )
 
       network->tick();
 
-      const Network::NetworkException *exn = network->get_send_exception();
-      if ( exn ) {
-        overlays.get_notification_engine().set_network_exception( *exn );
+      string & send_error = network->get_send_error();
+      if ( !send_error.empty() ) {
+        overlays.get_notification_engine().set_network_error( send_error );
+	send_error.clear();
       } else {
-        overlays.get_notification_engine().clear_network_exception();
+        overlays.get_notification_engine().clear_network_error();
       }
     } catch ( const Network::NetworkException &e ) {
       if ( !network->shutdown_in_progress() ) {
-        overlays.get_notification_engine().set_network_exception( e );
+        overlays.get_notification_engine().set_network_error( e.what() );
       }
 
       struct timespec req;
