@@ -39,28 +39,26 @@ namespace Terminal {
   /* variables used within a new_frame */
   class FrameState {
   public:
-    int x, y;
-    bool force_next_put;
     std::string str;
 
     int cursor_x, cursor_y;
     Renditions current_rendition;
+    bool cursor_visible;
 
-    Framebuffer last_frame;
+    const Framebuffer &last_frame;
 
-    FrameState( const Framebuffer &s_last )
-      : x(0), y(0),
-	force_next_put( false ),
-	str(), cursor_x(0), cursor_y(0), current_rendition( 0 ),
-	last_frame( s_last )
-    {
-      str.reserve( 1024 );
-    }
+    FrameState( const Framebuffer &s_last );
 
+    void append( char c ) { str.append( 1, c ); }
+    void append( size_t s, char c ) { str.append( s, c ); }
+    void append( wchar_t wc ) { Cell::append_to_str( str, wc ); }
     void append( const char * s ) { str.append( s ); }
-    void appendstring( const std::string &s ) { str.append( s ); }
+    void append_string( const std::string &append ) { str.append(append); }
 
+    void append_cell(const Cell & cell) { cell.print_grapheme( str ); }
     void append_silent_move( int y, int x );
+    void append_move( int y, int x );
+    void update_rendition( const Renditions &r, bool force = false );
   };
 
   class Display {
@@ -76,15 +74,11 @@ namespace Terminal {
 
     bool has_title; /* supports window title and icon name */
 
-    int posterize_colors; /* downsample input colors >8 to [0..7] */
-
     const char *smcup, *rmcup; /* enter and exit alternate screen mode */
 
-    void put_cell( bool initialized, FrameState &frame, const Framebuffer &f ) const;
+    bool put_row( bool initialized, FrameState &frame, const Framebuffer &f, int frame_y, const Row &old_row, bool wrap ) const;
 
   public:
-    void downgrade( Framebuffer &f ) const { if ( posterize_colors ) { f.posterize(); } }
-
     std::string open() const;
     std::string close() const;
 
